import { Component } from 'react'
import { View, Swiper, SwiperItem, Image, Textarea, Text, ScrollView, RichText } from '@tarojs/components'
import Taro, { getCurrentInstance } from '@tarojs/taro'
import tool from '../../common/tool'
import * as api from '../../service/index'
import './index.less'

export default class Task extends Component {
  $instance = getCurrentInstance()
  state = {
    userInfo: {},
    taskList: [],
    vip_list: [],//会员列表
    shop_list: [],//充值列表
    ext_info: {},//注意事项
    isIos: false,
  }

  componentWillMount () {
    if (tool.notAndroid()) {
      this.setState({
        isIos: true
      })
    }
    this.getCoinTaskList()
    this.getShopList()
    this.getUserInfo()
  }
  onShareAppMessage = (res) => {
    let shareData = {
      title: tool.shareTitle,
      path: `/pages/index/index?c=${Taro.getStorageSync('user_id')}&s=rightTap`,
      imageUrl: tool.shareImg
    }
    return shareData;
  }
  //获取任务列表
  getCoinTaskList () {
    api.getCoinTaskList().then(res => {
      this.setState({
        taskList: res.data
      })
    })
  }
  getUserInfo () {
    api.getUserInfo().then(res => {
      if (res.code == 200) {
        this.setState({
          userInfo: res.data
        })
      }
    })
  }
  getShopList () {
    api.getShopList().then(res => {
      if (res.code == 200) {
        this.setState({
          vip_list: res.data.vip_list,
          shop_list: res.data.shop_list,
          ext_info: res.data.ext_info,
        })
      }
    })
  }
  finishTask (item, index) {
    if (item.status != 0) return
    if (item.btn_title == '去点赞') {
      Taro.switchTab({
        url: '/pages/collection/index'
      })
      return
    }
    if (item.btn_title == '邀请') {
      Taro.navigateTo({
        url: '/moduleA/pages/invite/index'
      })
      return
    }
    api.finishTask({ task_id: item.id }).then(res => {
      if (res.code == 200) {
        this.getUserInfo()
        this.getCoinTaskList()
      }
    })
  }
  toShareCnt () {
    Taro.navigateTo({
      url: '/moduleA/pages/shareCnt/index'
    })
  }
  //充值
  toOrder (item, index) {
    let that = this
    Taro.getSystemInfo({
      success: function (res) {
        Taro.showLoading({
          title: '加载中...',
        })
        api.createOrder({ shop_id: item.id }).then(res => {
          if (res.code == 200) {
            Taro.hideLoading()
            Taro.requestPayment({
              timeStamp: res.data.timeStamp,
              nonceStr: res.data.nonceStr,
              package: res.data.package,
              signType: res.data.signType,
              paySign: res.data.paySign,
              success: function (res) {
                // console.log(res);
                //添加2s延迟在刷新数据
                setTimeout(() => {
                  that.getUserInfo()
                }, 2000);
              },
              fail: function (err) {
                try {
                  const res = Taro.getSystemInfoSync()
                  console.log(res, 'iphone=====');
                } catch (e) {
                }
              }
            })
          }
        })
      }
    })

  }
  //会员充值
  memberToOrder (item, index) {
    if (item.has_buy == 1) return
    let that = this
    Taro.getSystemInfo({
      success: function (res) {
        Taro.showLoading({
          title: '加载中...',
        })
        api.createOrder({ shop_id: item.id }).then(res => {
          if (res.code == 200) {
            Taro.hideLoading()
            Taro.requestPayment({
              timeStamp: res.data.timeStamp,
              nonceStr: res.data.nonceStr,
              package: res.data.package,
              signType: res.data.signType,
              paySign: res.data.paySign,
              success: function (res) {
                // console.log(res);
                //添加2s延迟在刷新数据
                setTimeout(() => {
                  this.getCoinTaskList()
                  this.getShopList()
                  this.getUserInfo()
                }, 2000);
              },
              fail: function (err) {
                try {
                  const res = Taro.getSystemInfoSync()
                  console.log(res, 'iphone=====');
                } catch (e) {
                }
              }
            })
          }
        })
      }
    })

  }
  componentDidMount () { }
  //页面销毁之前调用
  componentWillUnmount () {
    //避免服务器攻击
  }


  render () {
    return (
      <View className='mine'>
        <View className='banner'>
          <View className='banner-left'>
            <View className='banner-title'>剩余点券：</View>
            <View className='banner-num'>{this.state.userInfo.total_coin}</View>
          </View>
          <View className='banner-right'>
            <Image className='banner-img' src='https://we-spa.oss-cn-shenzhen.aliyuncs.com/total_picture/1679465544881.png'></Image>
          </View>
        </View>
        <View className='content'>
          {
            this.state.taskList.map((item, index) => (
              <View className='register' key={index}>
                <View className='register-left'>
                  <View className='register-title'><Image className='register-icon' src={item.img} ></Image> {item.title}</View>
                  <View className='register-tips'>{item.sub_title}</View>
                </View>
                <View className='register-right' style={item.status == 0 ? '' : 'background-image: linear-gradient(-45deg, #0c1419 0%, #3b3d3e 100%);opacity: 0.5;'} onClick={e => (this.finishTask(item, index))}>{item.btn_title}</View>
              </View>
            ))
          }
          {
            //分销用户
            this.state.userInfo.p_user_id == 1 &&
            <View className='register'>
              <View className='register-left'>
                <View className='register-title'><Image className='register-icon' src='http://video-img.fyshark.com/1680060033169%E5%88%86%E9%94%80.png' ></Image> 分销明细</View>
                {/* <View className='register-tips'>{item.sub_title}</View> */}
              </View>
              <View className='register-right' onClick={e => (this.toShareCnt())}>查看</View>
            </View>
          }
          {
            !this.state.isIos &&
            < View >
              <View className='contact-title'>
                点券充值
              </View>
              <ScrollView
                className='recharge'
                scrollX
              >
                {
                  this.state.shop_list.map((item, index) => (
                    <View className='recharge-info' key={index}>
                      <View className='recharge-icon'><Image className='recharge-img' src='https://we-spa.oss-cn-shenzhen.aliyuncs.com/total_picture/1679457146188.png'> </Image></View>
                      <View className='recharge-tips'>{item.sub_title}</View>
                      <View className='recharge-title'>{item.title}</View>
                      <View className='recharge-tag'>{item.tag}</View>
                      <View className='recharge-button' onClick={e => (this.toOrder(item, index))}>充值</View>
                    </View>
                  ))
                }
              </ScrollView>
              <View className='contact-title'>
                会员充值
              </View>
              <View className='member-list'>
                {
                  this.state.vip_list.map((item, index) => (
                    <View className='member-info' key={index}>
                      <View className='member-top'>
                        <RichText className='member-title' nodes={item.title}></RichText>
                        <View className='member-tag'>{item.tag}</View>
                      </View>
                      <View className='member-bottom'>
                        <View className='member-bottom-left'>
                          <RichText nodes={item.sub_title}></RichText>
                        </View>
                        <View className='member-button' style={item.has_buy == 1 ? 'background-image: linear-gradient(-45deg, #0c1419 0%, #3b3d3e 100%);opacity: 0.5;' : ''} onClick={e => (this.memberToOrder(item, index))}>{item.has_buy == 1 ? '已开通' : '开通'}</View>
                      </View>
                    </View>
                  ))
                }
              </View>
            </View>
          }
          {
            this.state.isIos &&
            <View className='ios-tips'>
              ios暂不支持充值
            </View>
          }
        </View>
      </View >
    )
  }
}
